#DEFINE _DQ    Chr(34)
#DEFINE _CR    Chr(13)

* --------------------------------------------------------------------    
* -- This program is a simple illustration of how to send JSON to a 
* -- Servoy web service. It contains NEARLY NO ERROR HANDLING, and in the real
* -- world should use classes and so on!
* -- Remember to modify the change variables section below to reflect your
* -- own setup.
* --
* -- Uses built-in Windows WinHTTP library which AFAIK is there back
* -- to Windows 98. You could also use an alternative like that 
* -- provided by West Wind software.
* --
* -- Since this is simple I build the JSON myself, for more complex
* -- requirements you could use Craig Boyd's free JSON FLL, or again
* -- the solutions from West Wind Software and others.
* --
* -- We use JSON because it suits this type of data, and the Servoy
* -- side can de-serialise it into Javascript objects very easily. 
* --
* -- Alan Bourke 2012 - please modify and distribute freely.
* --
* --------------------------------------------------------------------    

    Close All
    Clear All
    Clear
    Set Memowidth To 120
    
    * ---------------------------------------------------------------------------    
    * -- Change variables - should be changed to reflect your own Servoy server!
    * ---------------------------------------------------------------------------    
    lcSvyServer        = "MYSERVOYSERVER"
    lnSvyPort          = 8080
    * ---------------------------------------------------------------------------    

    * --------------------------------------------------------------------    
    * -- Variables 
    * --------------------------------------------------------------------    
    lcSvySolution      = "vfpwebservicedemo"    && -- The Servoy solution name.
    lcSvyService       = "ws_updateusers"          && -- The Servoy web service.
    loCol = CreateObject("Collection")
    loHTTP = Createobject("WinHttp.WinHttpRequest.5.1")
    
    * --------------------------------------------------------------------    
    * -- Main functionality starts here.
    * --------------------------------------------------------------------    

    * -- Timeouts are Resolve, Connect, Send, Receive in ms.
    loHTTP.setTimeouts(10000,10000,300000,300000)

    * -- Build the URL that we will be hitting to send information to Servoy.    
    lcURL = "http://" + Alltrim(lcSvyServer) + ":" + Alltrim(Str(lnSvyPort, 5, 0)) + "/"
    lcURL = lcURL + "servoy-service/rest_ws/" + Alltrim(lcSvySolution) + "/"
    lcURL = lcURL + Alltrim(lcSvyService)

    * -- Get our sample data.
    Use customers In 0 

    * -- Process changed records. We're building a collection of objects
    * -- that represent changed customers which we will then stringify.
    Scan For changed

        Scatter Name loChanged   
        loCol.Add(loChanged)

    EndScan
    
    * -- Convert our collection of changed customers to JSON.
    lcJSON = Stringify(loCol)

    * -- Attempt to send our JSON to the Servoy web service we specified.    
    * -- We use a HTTP PUT to do this. This will automatically pass the 
    * -- data we send with loHTTP.Send() to a Servoy function called
    * -- ws_create, on the Servoy server and service we specify with
    * -- the URL we build above.
    loHTTP.Open("PUT", lcURL)
    llSendError = .f.

    * -- Attempt to send the data. This might result in a HTTP-level
    * -- problem such as the server being unavailable.
    lcRet = "OK"
    Try

        loHTTP.Send(lcJSON)

    Catch To loHTTPException
            
        lcLastResponseText = loHTTPException.Message
        lnLastStatus = 408
        llSendError = .t.
        lcRet = "ERROR"
            
    EndTry

    * -- Was there a HTTP-level error?
    If !llSendError
        
        * -- We sent the HTTP OK, but did Servoy find some problem
        * -- with the data (an invalid customer code or something?)
        * -- and reject it?
        lcLastResponseText = CleanResponse(loHTTP.ResponseText)        
        lnLastStatus = loHTTP.Status
                
        If loHTTP.Status = 200
            lcRet = "OK"
        Else
            lcRet = "ERROR"
        Endif    
    
    Endif

    * -- So when we get to here:
    * --
    * -- lcRet will be OK or ERROR
    * --
    * -- lcLastResponseText will be a HTTP-level error.
    * --
    * -- lnLastStatus will be the last HTTP status code.
    * --
    ? lcRet
    ? lcLastResponseText
    ? lnLastStatus
    
Return

* --------------------------------------------------------------------    
* -- Stringify the collection of customers to JSON. I am rolling my own,
* -- there are also third-party libraries around that might work.
* --
* -- Will produce something like:
* --{
* --    "name": "customers",
* --        "items": 
* --        [
* --            {
* --                "acc_code": "IM0293",
* --                "acc_name": "Internet Marketing Council",
* --            }
* --,
* --            {
* --                "acc_code": "PP0399",
* --                "acc_name": "Paul & Paul Hairdressers",
* --            }
* --        ]
* --}
* --------------------------------------------------------------------    
Function Stringify(loCustomerCollection As Collection)
Local lCS, loCustomer, lnCust

    lcS = ""
    lnCust = 1
    
    lcS= "{" + _CR
    lcS = lcS + [    "name": "customers",] + _CR
    lcS = lcS + [        "items": ] + _CR
    lcS = lcS + "        [" + _CR
    
    For Each loCustomer In loCustomerCollection

        lcS = lcS + "            {" + _CR
        lcs = lcs + [                "acc_code": ] + SerialiseString(Alltrim(loCustomer.acc_code)) + "," + _CR
        lcs = lcs + [                "acc_name": ] + SerialiseString(Alltrim(loCustomer.acc_name)) + "," + _CR
        lcs = lcs + [            }] + _CR
        lcS = lcS + Iif(lnCust<loCustomerCollection.Count, "," + _CR, "")
    
        lnCust = lnCust + 1
        
    EndFor
    
    lcS = lcS + "        ]" + _CR
    lcS = lcS + [}]

Return lcS

Function SerialiseString(lcStringToEscape)

    * Strtran(x,Chr(34),"\"+Chr(34))    
    If Atc(Chr(92), lcStringToEscape) > 0
        lcStringToEscape = Strtran(lcStringToEscape,"\", "\\")
    Endif
    If Atc(Chr(10), lcStringToEscape) > 0
        lcStringToEscape = Strtran(lcStringToEscape,Chr(10), "\n")
    Endif
    If Atc(Chr(13), lcStringToEscape) > 0
        lcStringToEscape = Strtran(lcStringToEscape,Chr(13), "\r")
    Endif
    
    Return ["] + lcStringToEscape + ["]


Return lcStringToEscape

Function CleanResponse(lcText as String) as String
Return Chrtran(lcText, ["'], "")
Endfunc
